
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include "lseparable.h"
#include "odicon.h"

#define MAX(a,b) (((a)>(b)) ? (a) : (b))

static void output_matrix(char *name, const int dim, int **aa) {
    int i, j;

    printf("%s=\n", name);
    for (i=1; i<dim; i++) {  /* we ignore aa[0][0] */
        printf("{");
        for (j=0; j<i; j++) {
            /*printf("[%d,%d]%d ", i, j, aa[i][j]);*/
            printf("%d, ", aa[i][j]);
        }
        printf("},\n");
    }
    printf(";\n");
}

static void output_vector(char *name, int dim, int a[]) {
    int i;

    if (a == NULL) return;
    printf("%s= ", name);
    for (i=0; i<dim; i++) {
        /*printf("[%d]%d ", i, a[i]);*/
        printf("%d, ", a[i]);
    }
    printf(";\n");
}

void lseparable_output(struct lseparable_st *p) {
    printf("S = 0 .. %d;\n", p->dim-1);
    output_matrix("aa", p->dim, p->aa);
    output_matrix("bb", p->dim, p->bb);
    output_vector("a",  p->dim, p->a);
    output_vector("b",  p->dim, p->b);
    output_vector("c",  p->dim, p->c);
    printf("\n");
}

struct lseparable_st *lseparable_make(const int dim) {
    int i, j;
    struct lseparable_st *p;

    p = xmalloc(sizeof(*p));
    p->dim = dim;
    p->aa = imatrix_new(dim, dim);
    p->bb = imatrix_new(dim, dim);
    p->a  = ivector_new(dim);
    p->b  = ivector_new(dim);
    p->c  = ivector_new(dim);

    for (i=0; i<dim; i++) {
        for (j=0; j<i; j++) {
            p->aa[i][j] = irandom(0, 1000);
            p->bb[i][j] = irandom(-1000, 1000);
        }
        p->a[i] = irandom(1, 1000);
        p->b[i] = irandom(-1000, 1000);
        p->c[i] = irandom(-1000, 1000);
    }
    return p;
}

/* lseparable(x) = sum(aa[i][j] * (x[i]-x[j])^2 + bb[i][j] * (x[i]-x[j]), i<j)
                 + sum(a[i] * (x[i]-c[i])^2 + b[i] * (x[i]-c[i]), i) */
double lseparable(const int dim, double x[], struct lseparable_st *p) {
    int i, j;
    double r = 0;

    assert(p->dim == dim);
    for (i=0; i<dim; i++) {
        double z;
        for (j=0; j<i; j++) {
	    z = x[i] - x[j];
            if (p->aa[i][j] == 0) {
                r += exp(p->bb[i][j] * z / dim / dim / dim / dim);
            } else {
                r += p->aa[i][j] * z * z
                    +p->bb[i][j] * z;
            }
        }
        z = x[i] - p->c[i];
        r += p->a[i] * z * z + p->b[i] * z;
    }
    return r;
}

double simple_lseparable(const int dim, double x[], struct lseparable_st *p) {
    int i, j;
    double r = 0;

    assert(p->dim == dim);
    for (i=0; i<dim; i++) {
        for (j=0; j<i; j++) {
            double z = x[i] - x[j];
            r += p->aa[i][j] * z * z
                +p->bb[i][j] * z;
        }
    }
    return r;
}

double complex_lseparable(const int dim, double x[], struct lseparable_st *p) {
    int i, j;
    double r = 0;

    assert(p->dim == dim);
    for (i=1; i<dim; i++) {
        double z;
        for (j=1; j<i; j++) {
            z = x[i] - x[j];
            if (p->aa[i][j] == 0) {
                r += exp(p->bb[i][j] * z / dim / dim / dim / dim);
            } else {
                r += p->aa[i][j] * z * z
                    +p->bb[i][j] * z;
            }
        }
        z = x[i] - p->c[i];
        r += p->a[i] * z * z + p->b[i] * z;
    }
    r += 100 * MAX( 0, fabs(x[0] - x[1]) -3);
    return r;
}
